 /*****************************************************************************
  *                                                                           
  *  Copyright (c) 2008 Johannes Hermen www.der-frickler.net                   
  *                                                                           
  *                                                                           
  *  This library is free software; you can redistribute it and/or modify it  
  *  under the terms of the GNU Lesser General Public License as published    
  *  by the Free Software Foundation; either version 2 of the License, or     
  *  (at your option) any later version.                                      
  *                                                                           
  *  This software is distributed in the hope that it will be useful, but     
  *  WITHOUT ANY WARRANTY; without even the implied warranty of               
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        
  *  Lesser General Public License for more details.                          
  *                                                                           
  *  You should have received a copy of the GNU Lesser General Public         
  *  License along with this library; if not, write to the Free Software      
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA  
  *                                                                           
  *****************************************************************************
  *                                                                           
  *  BEEPER TINY1      Clock 9,6Mhz / 8                                       
  *  Toggles pin 3&4 to HIGH if signal length is > 1,7ms                      
  *                                                                           
  *  PINOUT: Servo Signal IN: PIN 5 (PB0)                                     
  *          Beeper OUT: PIN 2 (PB3) & PIN 3 (PB4)                            
  *                                                                           
  *****************************************************************************/

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>

uint8_t toggle = 1;
uint8_t new = 1;
uint8_t beeping;
uint16_t length;

// on timer0 overflow
ISR (TIM0_OVF_vect)	
{
	length += 255;
	// if no signal for some time -> enable beep
	if (length > 3000) {
		//PORTB = 8; 	
		beeping = 1;
	}
}

// on PB0 pinchange
ISR (PCINT0_vect)
{
  if ((PINB & 1)==1) 	// check 1st bit of PORTB (pin 5) got HIGH
  {
	length = 0;
    TCNT0 = 0; 			// reset timer 
    TCCR0B = 3;			// start timer (prescaler :64)
  }
  else {	// check 1st bit of PORTB (pin 5) got LOW
	TCCR0B = 0; 		// stop timer0
	if (TCNT0 >32) {	// if high was longer than 1,5ms
		
		if (new == 1) {
			if (toggle == 1) { 
				toggle = 0;
				beeping = 1;
			} else {
				toggle = 1;
				beeping = 0;
			}
		}
		new = 0;
	} else {
		new = 1;
	}
	TCNT0 = 0;  // reset timer 
	TCCR0B = 3;	// start timer (prescaler :64)
  }
}

int main (void)
{

  	DDRB = 254;	// portb PB0 as input, rest as output
	PORTB = 0; 	// portb to 0	

  	TCCR0B = 0; // stop timer0
  	TIMSK0 = 2; // timer0 Overflow Interrupt Enable
  	TCNT0 = 0; // timer startval = 0 


	GIMSK = 32;	// enable Ext Interrupt on PinChange
  	PCMSK = 1; 	// set Ext Interrupt on PB0 (pin 5)

  	sei(); 	// Interrupts on.

	uint16_t j; // define a wait counter variable

	while(1)
	{
		if (1 == beeping) { // blink whith short duty cycle
			PORTB = 24;
			j=10000;
            while(0<j--); 
			PORTB = 0;
			j=50000;
            while(0<j--); 
		} else {  			// set port to 0
			PORTB = 0;
		}
	}
}
